#include "fs/tmpfs/FileSystem.h"
#include <luna/Alloc.h>
#include <luna/Ignore.h>

namespace TmpFS
{
    Result<SharedPtr<VFS::FileSystem>> FileSystem::create()
    {
        SharedPtr<FileSystem> fs = TRY(adopt_shared(new (std::nothrow) FileSystem()));
        SharedPtr<VFS::Inode> root = TRY(fs->create_dir_inode());
        fs->set_root(root);
        return (SharedPtr<VFS::FileSystem>)fs;
    }

    Result<SharedPtr<VFS::Inode>> FileSystem::create_file_inode()
    {
        SharedPtr<FileInode> inode = TRY(make_shared<FileInode>());
        inode->set_fs(*this, {});
        inode->set_inode_number(m_next_inode_number, {});
        TRY(m_inodes.try_append(inode));
        m_next_inode_number++;
        return (SharedPtr<VFS::Inode>)inode;
    }

    Result<SharedPtr<VFS::Inode>> FileSystem::create_dir_inode()
    {
        SharedPtr<DirInode> inode = TRY(make_shared<DirInode>());
        TRY(inode->add_entry(inode, "."));

        inode->set_fs(*this, {});
        inode->set_inode_number(m_next_inode_number, {});
        TRY(m_inodes.try_append(inode));
        m_next_inode_number++;
        return (SharedPtr<VFS::Inode>)inode;
    }

    void FileSystem::set_root(SharedPtr<VFS::Inode> root)
    {
        m_root_inode = root;
    }

    Result<SharedPtr<VFS::Inode>> DirInode::find(const char* name) const
    {
        for (const auto& entry : m_entries)
        {
            if (!strcmp(name, entry.name.chars())) return entry.inode;
        }

        return err(ENOENT);
    }

    Result<void> DirInode::add_entry(SharedPtr<VFS::Inode> inode, const char* name)
    {
        Entry entry = { inode, TRY(OwnedStringView::from_string_literal(name)) };

        TRY(m_entries.try_append(move(entry)));

        return {};
    }

    Result<SharedPtr<VFS::Inode>> DirInode::create_file(const char* name)
    {
        auto inode = TRY(m_fs->create_file_inode());

        TRY(add_entry(inode, name));

        return inode;
    }
}
